/***********************************************************
* LCD Interface Library for the Hitachi HD44780 controller *
*           (C) 2009 by Marius Petrescu, YO2LOJ            *
*       Released under the GNU General Public License      *
***********************************************************/
#ifndef _INCLUDE_LCD_H_
#define _INCLUDE_LCD_H_

#include <avr/pgmspace.h>

/*-------------- Macros and Defines --------------*/

/***********************************************/
/* These macros are architecture dependant     */
/***********************************************/

/* E, R/W and RS are at PORTD(0x12) bits 1-4-6 */
/* macro to set port direction */
#define LCD_SET_CTL_WR()   (DDRD |= 0x52)

/* macros to set and reset the specific bits */
#define LCD_SET_E()     __asm__ __volatile__ ("sbi 0x12,1\n")
#define LCD_CLEAR_E()   __asm__ __volatile__ ("cbi 0x12,1\n")
#define LCD_SET_RW()    __asm__ __volatile__ ("sbi 0x12,4\n")
#define LCD_CLEAR_RW()  __asm__ __volatile__ ("cbi 0x12,4\n")
#define LCD_SET_RS()    __asm__ __volatile__ ("sbi 0x12,6\n")
#define LCD_CLEAR_RS()  __asm__ __volatile__ ("cbi 0x12,6\n")

/* data port is PORTB bits 4..7 */
#define LCD_PORT        PORTB
/* macro to set port direction */
#define LCD_SET_DATA_WR()	(DDRB |= 0xF0)
#define LCD_SET_DATA_RD()	(DDRB &= 0x0F)
/* macro to send lower nibble to port */
#define LCD_SEND_NIBBLE(value)  (PORTB = (PINB & 0x0F)|((value<<4) & 0xF0))

/*-------------- Variable references --------------*/

extern char lcd_line_cmd[];	// for lcd_goto() macro

/*-------------- Function Prototypes --------------*/

/* Initialise sequence for the lcd */
/* MUST BE CALLED BEFORE ANY OTHER FUNCTIONS */
void lcd_init(void);				 

/* Send command byte to lcd */
void lcd_sendcmd(char cmd);

/* Send character to lcd */
void lcd_sendchar(char ch);

/* Clear the lcd */
void lcd_clear(void);

/* PROGMEM enabled output null terminated string to lcd */
void lcd_puts_P(const prog_char *str);				  

/* Macro: Go to specified position on lcd (0 based) */
#define lcd_goto(line, row) (lcd_sendcmd(0x80 | lcd_line_cmd[line] | row))

/* delay routine, param in usec */
void lcd_delay(long int d);

#endif